/* $Id: kmo_illumination_flat-test.c,v 1.1 2013-10-21 13:44:55 aagudo Exp $
 *
 * This file is part of the KMOS Library
 * Copyright (C) 2002-2006 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

/*
 * $Author: aagudo $
 * $Date: 2013-10-21 13:44:55 $
 * $Revision: 1.1 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include <cpl.h>
#include "kmclipm_functions.h"

#include "kmo_dfs.h"
#include "kmo_test_create_data.h"

const char      *path_recipe        = "kmo_illumination_flat/";

const char      *valid_files[]      = {"flat_on_123.fits",
                                       "flat_on_231.fits",
                                       "flat_on_312.fits"};

/**
    @defgroup kmo_illumination_flat   kmo_illumination_flat unit tests
*/

/**
  @brief    Test of kmo_illumination_flat recipe.
  @param    argc   the number of parameters
  @param    argv   the parameter list

  Call @c kmo_illumination_flat-test @c --generate when only the test data for this
  recipe should be generated.
  Call @c kmo_illumination_flat-test @c --no-gen when only the tests should be
  executed and no data should be generated.
 */
int main(int argc, char *argv[])
{
    float   tol     = 0.01;

    char    test_path[256],
            file_path[256],
            op1_file[256],
            op2_file[256],
            op3_file[256],
            esorex_command[512];

    const char *sof_path = "test_illumination.sof";

    cpl_propertylist *h = NULL;

    cpl_test_init("<kmos-spark@mpe.mpg.de>", CPL_MSG_WARNING);

    /* ----- check if test data exists ----- */
    if (kmo_test_file_exists(test_global_path_test_data) == FALSE) {
        mkdir(test_global_path_test_data, 0777);
        kmo_test_create_pipeline_data();
    }

    cpl_msg_info(cpl_func, "Testing kmo_illumination_flat with real "
                           "pipeline data.");
    sprintf(test_path, "%s%s", test_global_path_test_data, "pipeline/");

    if (kmo_test_file_exists(test_path) == FALSE) {
        mkdir(test_path, 0777);
    }

    // get rid of warning...
    if ((argc == 5) && (strcmp(argv[1], "xxx") != 0)) {
    }

    FILE *fd = fopen("log_kmo_illumination_flat.txt", "w");
    fprintf(fd, "%s\n", " ");
    fclose(fd);

    //
    //  PIPELINE TEST (must be called after kmo_wave_cal-test, since the
    //  following recipes rely on this output)
    //
    kmo_test_cat_strings(op1_file, test_path, valid_files[0]);
    kmo_test_cat_strings(op2_file, test_path, valid_files[1]);
    kmo_test_cat_strings(op3_file, test_path, valid_files[2]);

    // create sof-file
    FILE *fh = fopen(sof_path, "w");
    fprintf (fh,
             "%s                           FLAT_SKY_FLAT\n"
             "%s                           FLAT_SKY_FLAT\n"
             "%s                           FLAT_SKY_FLAT\n"
             "xcal_KKK.fits                XCAL\n"
             "ycal_KKK.fits                YCAL\n"
             "lcal_KKK.fits                LCAL\n"
             "ref_data/kmos_wave_band.fits WAVE_BAND\n",
             op1_file, op2_file, op3_file);
    fclose(fh);

    // create esorex-command
    sprintf(esorex_command,
            "esorex --suppress-prefix=TRUE kmo_illumination_flat "
            "--imethod=NN --b_start=1.948 --b_end=2.351 %s", sof_path);

    kmo_get_pipe_command(esorex_command,
                         "log_kmo_illumination_flat.txt", sof_path, TRUE);

    // execute esorex-command
    cpl_test_eq(0, system(esorex_command));

    // checking output
    strcpy(file_path, "illum_corr_KKK");
    cpl_test_abs(kmo_test_esorex_data(file_path, 1), 87.0973, tol);
    strcat(file_path, ".fits");

    h = kmclipm_propertylist_load(file_path, 0);
    cpl_test_abs(cpl_propertylist_get_double(h, QC_SPAT_UNIF),
                 0.00265038, tol);
    cpl_test_abs(cpl_propertylist_get_double(h, QC_SPAT_MAX_DEV),
                 0.00448508 , tol);
    cpl_test_abs(cpl_propertylist_get_double(h, QC_SPAT_MAX_NONUNIF),
                 0.00131134, tol);
    cpl_propertylist_delete(h); h = NULL;

    return cpl_test_end(0);
}

/** @} */
